#include <time.h>
#include <stdio.h>
#ifndef _WIN32
#include <unistd.h>
#endif
#include <FL/Fl.H>
#include <FL/Fl_Window.H>
#include <FL/Fl_Single_Window.H>
#include <FL/Fl_Pixmap.H>
#include <FL/Fl_Box.H>

/************************************************************************

    nixieclock - a nixie desktop timepiece
    erco@netcom.com 1.00 11/21/00
    Copyright (C) 2000 Greg Ercolano

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*************************************************************************/

// THE DIGIT XPM FILES
#include "xpms/dots.xpm"
#include "xpms/d0.xpm"
#include "xpms/d1.xpm"
#include "xpms/d2.xpm"
#include "xpms/d3.xpm"
#include "xpms/d4.xpm"
#include "xpms/d5.xpm"
#include "xpms/d6.xpm"
#include "xpms/d7.xpm"
#include "xpms/d8.xpm"
#include "xpms/d9.xpm"

#define UPDATESECS	20.0

// QUICKIE CLASS OVERLOADS Fl_Window EVENT HANDLER
class MyWindow : public Fl_Single_Window
{
    int handle(int e);
public:
    MyWindow(int x,int y,int w,int h,const char *l=0):Fl_Single_Window(x,y,w,h,l) { }
    MyWindow(int w,int h,const char *l=0):Fl_Single_Window(w,h,l) { }
};

MyWindow *mainwin;		// main window
Fl_Pixmap *pixmap_dots;		// nixie dots ':'
Fl_Pixmap *pixmap_digits[10];	// nixie digits '0-9'
Fl_Box    *dbox[5];		// digit boxes; HH:MM

// HANDLE EVENTS IN THE WINDOW
int MyWindow::handle(int e)
{
    static int xoff = 0, yoff = 0;
    switch ( e )
    {
        // DOWNCLICK IN WINDOW CREATES CURSOR OFFSETS
	case FL_PUSH:
	    xoff = x() - Fl::event_x_root();
	    yoff = y() - Fl::event_y_root();
	    return(1);

	case FL_DRAG:
	    // DRAG THE WINDOW AROUND THE SCREEN
	    position(xoff + Fl::event_x_root(), yoff + Fl::event_y_root());
	    redraw();
	    return(1);
    }
    return(Fl_Single_Window::handle(e)); 
}

// UPDATE DIGITS WITH THE CURRENT TIME
void UpdateTime()
{
    // GET CURRENT TIME AS SEPARATE DIGITS
    time_t tim = time(NULL);
    struct tm *buf;
    buf = localtime(&tim);
    int h10 = buf->tm_hour / 10,
        h1  = buf->tm_hour % 10,
	m10 = buf->tm_min / 10,
	m1  = buf->tm_min % 10;

    // DEBUGGING
    // fprintf(stderr, "TIME IS: %d%d:%d%d\n", h10, h1, m10, m1);

    pixmap_digits[h10]->label(dbox[0]);
    pixmap_digits[h1 ]->label(dbox[1]);
    pixmap_dots->label(dbox[2]);
    pixmap_digits[m10]->label(dbox[3]);
    pixmap_digits[m1 ]->label(dbox[4]);
    mainwin->redraw();
}

// TIMER: UPDATE DIGITS ON EACH TICK
void Tick_CB(void *) 
{
    UpdateTime();
    Fl::add_timeout(UPDATESECS, Tick_CB);
}

int main(int argc, char **argv) 
{
    // BANNER
    //    (simply thrown away under windows)
    //
    fprintf(stderr, "nixieclock 1.00\n"
                    "(C) Copyright 2000 Greg Ercolano (erco@netcom.com).\n"
		    "This code is GPL'ed.\n\n");

#ifndef _WIN32
    // UNIX: RUN IN THE BACKGROUND
    switch ( fork() )
    {
	case -1: perror("fork"); exit(1); 	// ERROR
        case  0: break;				// CHILD
        default: exit(0);			// PARENT DISCONNECTS
    }
#endif

    int x = 10, y = 2, w = 23, h = 31, xstep = w + 2;

    // BUILD DIGIT PIXMAPS
    pixmap_digits[0] = new Fl_Pixmap(d0);
    pixmap_digits[1] = new Fl_Pixmap(d1);
    pixmap_digits[2] = new Fl_Pixmap(d2);
    pixmap_digits[3] = new Fl_Pixmap(d3);
    pixmap_digits[4] = new Fl_Pixmap(d4);
    pixmap_digits[5] = new Fl_Pixmap(d5);
    pixmap_digits[6] = new Fl_Pixmap(d6);
    pixmap_digits[7] = new Fl_Pixmap(d7);
    pixmap_digits[8] = new Fl_Pixmap(d8);
    pixmap_digits[9] = new Fl_Pixmap(d9);
    pixmap_dots      = new Fl_Pixmap(dots);

    // MAKE MAIN WINDOW
    mainwin = new MyWindow(x+(xstep*5)+x, h+(y*2), "nixieclock");
    {
	mainwin->color(0);
	mainwin->clear_border();

	// CREATE Fl_Box FOR EACH DIGIT POSITION AS CHILD OF WINDOW
	for ( int t=0; t<=4; t++ )
	{
	    dbox[t] = new Fl_Box(x, y, w, h);
	    dbox[t]->box(FL_NO_BOX);
	    x += xstep;
	}

	// UPDATE CURRENT TIME FOR FIRST REDRAW
	UpdateTime();
    }
    mainwin->end();
    mainwin->free_position();
    mainwin->show();

    // FLTK TIMER
    Fl::add_timeout(UPDATESECS, Tick_CB);
    return(Fl::run());
}
